/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wHMSTime.h"
#include <iomanip>
#include <sstream>

using namespace fwk;

HMSTime::HMSTime(double time)
{
    double secinmins = 60;
    double secinhours = 60 * secinmins;
    double secindays = 24 * secinhours;
    double secinmonths = 30 * secindays;
    double secinyears = 365 * secinmonths;

    years = static_cast<int>(time / secinyears);
    time -= years * secinyears;

    months = static_cast<int>(time / secinmonths);
    time -= months * secinmonths;

    days = static_cast<int>(time / secindays);
    time -= days * secindays;

    hours = static_cast<int>(time / secinhours);
    time -= hours * secinhours;

    mins = static_cast<int>(time / secinmins);
    secs = time - mins * secinmins;
}

std::string
HMSTime::str() const
{
    std::ostringstream st;
    if (years)
        st << years << "y";
    if (years || months)
        st << months << "mo";
    if (years || months || days)
        st << days << "d";
    if (years || months || days || hours)
        st << hours << "h";
    if (years || months || days || hours || mins)
        st << mins << "m";
    st << std::fixed << std::setprecision(2) << secs << "s";

    return st.str();
}
namespace fwk
{
FWK_API std::ostream &
operator<<(std::ostream &out, HMSTime const &obj)
{
    out << obj.str();
    return out;
}
}; // namespace fwk
