/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WIN32
#include <unistd.h>
#endif
#include "wTime.h"
#include "wHMSTime.h"

using namespace fwk;

namespace fwk
{

FWK_API std::ostream &
operator<<(std::ostream &out, Time const &obj)
{
    out << obj.hms();
    return out;
}

} // namespace fwk

std::string
Time::hms() const
{
    return HMSTime(sec()).str();
}

#ifdef WIN32

double
Int64Time::sec() const
{
    static __int64 freq = 10000000U;
    return time / static_cast<double>(freq);
}
#else

double
UnixClockTime::sec() const
{
    static clock_t freq = sysconf(_SC_CLK_TCK);
    return time / static_cast<double>(freq);
}

#endif

double
ClockTime::sec() const
{
    static clock_t freq = CLOCKS_PER_SEC;
    return time / static_cast<double>(freq);
}

double
DblTime::sec() const
{
    return time;
}
