#!/usr/bin/env python3
# -*- coding: utf-8; -*-

from katoptron.eigenvalues import *
import os
import fwk


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    n_sweeps_Q = 10
    damp_Q = 1.6

    n_sweeps_R = 10
    damp_R = 1.6

    omega = 0.5

    N = 32

    input_file_dir = file_dir + \
        '/../../workspace/katoptron_CMAME_tests_2_cubes_patch_convergence/1/'

    sample_per_rank = np.ceil((N / siz))
    i_min = rank*sample_per_rank
    i_max = i_min+sample_per_rank

    if i_min > N:
        i_min = N
    if i_max > N:
        i_max = N

    i_min = int(i_min)
    i_max = int(i_max)

    print('Rank ' + str(rank) + ' from sample ' + str(i_min) + ':' + str(i_max))

    for i in range(i_min, i_max):
        timers = fwk.Timers()

        filename_Q = input_file_dir + str(i) + '/ApCTC_mm.txt'
        filename_GT = input_file_dir + str(i) + '/B_mm.txt'
        filename_f = input_file_dir + str(i) + '/b_mm_after_bc.txt'

        timers['Read files'].start()
        Q = read_mm(filename_Q)
        Q = get_symmetrical_part(Q)
        GT = read_mm(filename_GT)
        G = GT.transpose()
        f = read_mm(filename_f, is_sparse=False)
        timers['Read files'].stop()

        timers['Compute R'].start()
        R = approx_Schur_complement(Q, G, GT)
        timers['Compute R'].stop()

        timers['Compute invR'].start()
        invR = Gauss_Seidel(R, n=n_sweeps_R, damping=damp_R)
        timers['Compute invR'].stop()

        print(timers)

        timers['Compute the inverse of Q'].start()
        invQ = Gauss_Seidel(Q, n=n_sweeps_Q, damping=damp_Q)
        timers['Compute the inverse of Q'].stop()

        print(timers)

        timers['Compute the preconditioned matrix'].start()
        AinvP = SIMPLE_preconditioned_matrix(
            Q, invQ, G, GT, R, invR, omega=omega)
        timers['Compute the preconditioned matrix'].stop()

        print(timers)

        tmp = np.zeros(AinvP.shape)
        for i in range(0, AinvP.shape[0]):
            tmp[i, i] = 1
            tmp[:, i] = AinvP.dot(tmp[:, i])

        print(AinvP.shape)

        w, v, cond = compute_eig_and_departure_from_orthogonality(tmp)
        print(cond)

        return
        timers['Compute the basis'].start()

        n = Q.shape[0]+R.shape[0]
        b = np.zeros((n,))
        b[0:len(f)] = f

        N = 2
        m_max = 6000
        V, W = compute_VW(AinvP, b, m=m_max, N=N)
        timers['Compute the basis'].stop()

        print(timers)

        timers['Compute the eigenvalues'].start()
        deltas, omegas = compute_deltas_omegas(V, W, b)
        timers['Compute the eigenvalues'].stop()

        print(timers)

        timers['Write the eigenvalues'].start()
        write_eigenvalues(deltas, work_dir+'/deltas_GS_GS_' + str(i) + '.txt')
        np.savetxt(work_dir+'/omegas_GS_GS_' + str(i) + '.txt', omegas)
        timers['Write the eigenvalues'].stop()

        f = open('timers_' + str(i) + '.txt', 'w')
        print(timers, file=f)
        f.closed


if __name__ == "__main__":
    main()
