#!/usr/bin/env python3
# -*- coding: utf-8; -*-




import katoptron as m
import tbox
import tbox.gmsh as gmsh
import katoptron.utilities as u
from fwk.testing import *
from katoptron.readers import read_Belos


def main():
    comm, rank, size = u.import_MPI()

    from PyTrilinos import Teuchos

    import shlex
    import subprocess
    import os
    import numpy as np

    geo_name = '2_cubes.geo'
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir, work_dir, comm, rank, size)

    ensemble_size = 1

    pbl = m.Problem(msh, comm)

    #E = np.linspace(0.5,1.5,ensemble_size)
    E = 1. * np.ones(ensemble_size)
    nu = 0.2 * np.ones(ensemble_size)
    k = 0.114
    d = 0.
    m.Medium(pbl, "Body 1", "test", E, nu, k, d)
    m.Medium(pbl, "Body 2", "test", E, nu, k, d)

    m.Dirichlet(pbl, "Surf 1 1", "Clamped", 1, 0., 1, 0., 1, 0., 1, 10.,
                ensemble_size)
    m.Dirichlet(pbl, "Surf 2 2", "Clamped", 1, 0., 1, 0., 1, -2., 1, 20.,
                ensemble_size)

    norm = tbox.Vector3d(0, 0, -1)
    cont = m.Contact(pbl, "Surf 1 2", "contact", norm)
    cont.setInitialyOpen()
    cont.setMaster(pbl, "Surf 2 1", norm)

    dx = np.zeros(ensemble_size)
    dy = 2. * np.ones(ensemble_size)
    dz = np.zeros(ensemble_size)
    dT = 10. * np.ones(ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 1000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList["MueLu xml file name"] = file_dir + '/direct_solver.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 3

    solverList['Write matrix and vectors'] = True
    solverList['Write txt files'] = True

    slv = m.IterativeSolver(pbl, solverList, 4, ensemble_size)

    nThreads = u.Initialize_Kokkos()

    slv.start()

    u.Finalize_Kokkos()

    tests = CTests()
    measured_total_CPU_cost = slv.getTimers()["total"].read().getReal().sec()

    non_zero_CPU_cost = (measured_total_CPU_cost > 0.)
    tests.add(CTest('non zero total CPU cost', non_zero_CPU_cost, True))

    iter_indices, residuals = read_Belos(work_dir + '/belos_out.txt', 1)

    nIterations = iter_indices[-1]
    has_converged = (solverList['Maximum Iterations'] > nIterations)
    tests.add(CTest('Has converged', has_converged, True))

    tag_name = "T measure"
    which_pos = 0
    which_dof = 3
    DOFperNode = 4
    name_sol = "x_mm.txt"

    pos_1, sol_1 = u.extract(msh, tag_name, DOFperNode, which_pos, which_dof,
                             ensemble_size, name_sol)

    measured_T_1 = sol_1[0]
    tests.add(CTest('Temperature 1', measured_T_1, 14.170731, 5e-5))

    tests.run()


if __name__ == "__main__":
    main()
