/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// SWIG input file of the 'mirrors' module

%feature("autodoc","1");

%module(docstring=
"'mirrorsw' module
(c) ULg - A&M",
directors="1",
threads="1"
) mirrorsw
%{

#include <string>
#include <sstream>
#include <typeinfo>
#include "mirrors.h"

#include "fwkw.h"
#include "tboxw.h"

#include "wProblem.h"
#include "wSolver.h"
#include "wMedium.h"
#include "wuDirichlet.h"
#include "wuNeumann.h"
#include "wTDirichlet.h"
#include "wTNeumann.h"
#include "wTSource.h"
#include "wMSurface.h"
#include "wANSYSSolution.h"

//Fix for missing SWIGPY_SLICE_ARG with some versions of swig.
#if PY_VERSION_HEX >= 0x03020000
# define SWIGPY_SLICE_ARG(obj) ((PyObject*) (obj))
#else
# define SWIGPY_SLICE_ARG(obj) ((PySliceObject*) (obj))
#endif

%}


%include "fwkw.swg"

// ----------- MODULES UTILISES ------------
%import "fwkw.i"
%import "tboxw.i"

// ----------- MIRRORS CLASSES ----------------
%include "mirrors.h"

%shared_ptr(mirrors::Medium);
%shared_ptr(mirrors::TDirichlet);
%shared_ptr(mirrors::TNeumann);
%shared_ptr(mirrors::uDirichlet);
%shared_ptr(mirrors::uNeumann);
%shared_ptr(mirrors::TSource);
%shared_ptr(mirrors::MSurface);


%immutable mirrors::Problem::msh; // avoid the creation of the setter method
%include "wProblem.h"
%immutable mirrors::Solver::tms; // avoid the creation of the setter method
%include "wSolver.h"

%pythonappend mirrors::Medium::Medium "self.thisown=0"
%include "wMedium.h"
%pythonappend mirrors::uDirichlet::uDirichlet "self.thisown=0"
%include "wuDirichlet.h"
%pythonappend mirrors::uNeumann::uNeumann "self.thisown=0"
%include "wuNeumann.h"

%pythonappend mirrors::TDirichlet::TDirichlet "self.thisown=0"
%include "wTDirichlet.h"
%pythonappend mirrors::TNeumann::TNeumann "self.thisown=0"
%include "wTNeumann.h"

%pythonappend mirrors::TSource::TSource "self.thisown=0"
%include "wTSource.h"

%pythonappend mirrors::MSurface::MSurface "self.thisown=0"
%include "wMSurface.h"

%pythonappend mirrors::ANSYSSolution::ANSYSSolution "self.thisown=0"
%include "wANSYSSolution.h"

// Instantiate some std templates
%include "std_vector.i"

namespace std {
   %template(std_vector_ANSYSSolution) std::vector<mirrors::ANSYSSolution*>;
   %template(std_vector_MirrorsProblem) std::vector<mirrors::Problem*>;
}

