/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLINE2_H
#define WLINE2_H

#include "tbox.h"
#include "wLazy.h"
#include "wElement.h"
#include "wCacheLine2.h"

namespace tbox
{

/**
 * @brief Linear line finite element
 * @authors Romain Boman, Adrien Crovato
 */
class TBOX_API Line2 : public Element
{
#ifndef SWIG
    friend class MemLine2;
    MemLine2 *pmem; ///< private memory of precalculated values
    inline MemLine2 &getMem() const;
    inline static CacheLine2 &getCache(int n);
    virtual double buildJ(size_t k) const override;
    virtual void buildGradientJ(size_t k, std::vector<double> &dDetJ) const override;
    virtual std::vector<double> buildGradientV(int dim) const override;

    virtual Eigen::VectorXd builds(std::vector<double> const &u, Fct0 const &f) override;
    virtual Eigen::VectorXd builds(std::vector<double> const &u, Fct1 const &f) override;

    virtual double computeGrad2(std::vector<double> const &u, size_t k) const override;
    virtual double computeV(std::vector<double> const &u, Fct0 const &fct) override;
    virtual double computeV() override;

    virtual std::vector<Eigen::Vector3d> computeTangents() const override;
    virtual std::vector<Eigen::Vector3d> buildTangents(size_t k) const override;
#endif

public:
    Line2(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Line2();
    virtual ELTYPE type() const override { return ELTYPE::LINE2; }

#ifndef SWIG
    virtual Mem &getVMem() const override;
    virtual Cache &getVCache() const override;
    virtual Eigen::Vector3d normal() const override;
    virtual std::vector<Eigen::Vector3d> gradientNormal() const override;
#endif
};

#include "wLine2.inl"

} // namespace tbox

#endif //WLINE2_H
